@@ -27,10 +27,6 @@
 #ifndef QUICHE_H
 #define QUICHE_H
 
-#if defined(__cplusplus)
-extern "C" {
-#endif
-
 #include <stdint.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -52,6 +48,10 @@ extern "C" {
 #define ssize_t SSIZE_T
 #endif
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 // QUIC transport API.
 //
 
@@ -124,6 +124,9 @@ enum quiche_error {
 
     // Error in key update.
     QUICHE_ERR_KEY_UPDATE = -19,
+
+    // The peer sent more data in CRYPTO frames than we can buffer.
+    QUICHE_ERR_CRYPTO_BUFFER_EXCEEDED = -20,
 };
 
 // Returns a human readable string with the quiche version number.
@@ -161,6 +164,9 @@ void quiche_config_verify_peer(quiche_config *config, bool v);
 // Configures whether to send GREASE.
 void quiche_config_grease(quiche_config *config, bool v);
 
+// Configures whether to do path MTU discovery.
+void quiche_config_discover_pmtu(quiche_config *config, bool v);
+
 // Enables logging of secrets.
 void quiche_config_log_keys(quiche_config *config);
 
@@ -172,6 +178,9 @@ int quiche_config_set_application_protos(quiche_config *config,
                                          const uint8_t *protos,
                                          size_t protos_len);
 
+// Sets the anti-amplification limit factor.
+void quiche_config_set_max_amplification_factor(quiche_config *config, size_t v);
+
 // Sets the `max_idle_timeout` transport parameter, in milliseconds, default is
 // no timeout.
 void quiche_config_set_max_idle_timeout(quiche_config *config, uint64_t v);
@@ -372,12 +381,18 @@ size_t quiche_conn_send_quantum_on_path(const quiche_conn *conn,
 
 
 // Reads contiguous data from a stream.
+// out_error_code is only set when STREAM_STOPPED or STREAM_RESET are returned.
+// Set to the reported error code associated with STOP_SENDING or STREAM_RESET. 
 ssize_t quiche_conn_stream_recv(quiche_conn *conn, uint64_t stream_id,
-                                uint8_t *out, size_t buf_len, bool *fin);
+                                uint8_t *out, size_t buf_len, bool *fin,
+                                uint64_t *out_error_code);
 
 // Writes data to a stream.
+// out_error_code is only set when STREAM_STOPPED or STREAM_RESET are returned.
+// Set to the reported error code associated with STOP_SENDING or STREAM_RESET. 
 ssize_t quiche_conn_stream_send(quiche_conn *conn, uint64_t stream_id,
-                                const uint8_t *buf, size_t buf_len, bool fin);
+                                const uint8_t *buf, size_t buf_len, bool fin,
+                                uint64_t *out_error_code);
 
 // The side of the stream to be shut down.
 enum quiche_shutdown {
@@ -542,6 +557,9 @@ typedef struct {
     // The number of received bytes.
     uint64_t recv_bytes;
 
+    // The number of bytes acked.
+    uint64_t acked_bytes;
+
     // The number of bytes lost.
     uint64_t lost_bytes;
 
